# ========================================================================================================
#
# John Pritchard
#
# Developed from esotk_fringe_obj_mask.py by Mark Neeser
#
# A python interactive window for use in the Austrian in-kind
# fringe-correction Reflex workflow.  This interactive window
# shows the results of the fringe-correction pipeline, allowing
# the User to compare the input image with the fringe-corrected
# image, and the fringe map.   From this interactive Actor the
# User can modify pipeline parameters and re-initiate the
# processing.
#
#
#
# ========================================================================================================

import os
import re
import sys
sys.path.append(os.path.dirname(__file__))
from esotk_giraf_gui import BaseDataPlotterManager, runInteractiveApp

try:
    # Import non-standard but needed modules. These must be as subset of what
    # the esotk_fringe_gui module tries to load.
    import reflex
    import pipeline_product
    import pipeline_display
    import matplotlib
    from giraf_plot_common import *

    class DataPlotterManager(BaseDataPlotterManager):

        # This function will read all the columns, images and whatever is needed
        # from the products. The variables , self.plot_x, self.plot_y, etc...
        # are used later in function plotProductsGraphics().
        def readFitsData(self, fitsFiles):
            # Control variable to check if the interesting files were found in
            # the input.
            self.science_frames = []
            self.err_frames = []
            self.raw_frames = []
            self.frameLabels = {}

            #Initialise the objects to read/display
            self.std_rbnspectra     = None
            self.fib_id = -1

            # Read all the products
            sf = {}
            ef = {}
            for frame in fitsFiles:
                if frame == '':
                    continue
                category = frame.category
                if category == 'STD_RBNSPECTRA':
                    HDUlist=pyfits.open(frame.name)
                    CRCM=HDUlist[0].header['CMETHOD']
                    HDUlist.close()
                    self.frameLabels[frame.name]=CRCM+': '+frame.name
                    sf[CRCM]=frame
                elif category == 'STD_RBNERRORS':
                    HDUlist=pyfits.open(frame.name)
                    CRCM=HDUlist[0].header['CMETHOD']
                    HDUlist.close()
                    self.frameLabels[frame.name]=CRCM+': '+frame.name
                    ef[CRCM]=frame
                elif category == 'STD':
                    self.raw_frames.append(frame)
            self.science_frames.append(sf['NONE'])
            del sf['NONE']
            self.science_frames.extend(sorted(sf.values(), key=lambda x: x.category, reverse=False))
            self.err_frames.append(ef['NONE'])
            del ef['NONE']
            self.err_frames.extend(sorted(ef.values(), key=lambda x: x.category, reverse=False))

            self.science_found = len(self.science_frames) > 0
            self.err_found = len(self.err_frames) > 0
            self.raw_found = len(self.raw_frames) > 0

            if self.science_found:
                frame = self.science_frames[0]
                self.science = pipeline_product.PipelineProduct(frame)
                self.std_rbnspectra        = PlotableSCI_RBNSPECTRA(self.science_frames[0])
                self.std_rbnspectra_extr   = PlotableExtractedScience(self.science_frames[0])
                self.esorex_log_file_name  = re.sub(".*/giraffe/" ,"../../", os.path.dirname(frame.name))+"/log_dir/esorex.log"

            if self.err_found:
                frame = self.err_frames[0]
                self.err = pipeline_product.PipelineProduct(frame)

            self.DPR_CATG = "unknown"
            self.DPR_TYPE = "unknown"
            self.DPR_TECH = "unknown"
            if self.raw_found:
                HDUlist=pyfits.open(self.raw_frames[0].name)
                self.DPR_CATG = HDUlist[0].header['HIERARCH ESO DPR CATG']
                self.DPR_TYPE = HDUlist[0].header['HIERARCH ESO DPR TYPE']
                self.DPR_TECH = HDUlist[0].header['HIERARCH ESO DPR TECH']
                HDUlist.close()

        def loadProducts(self, index):
            """
            Loading both science and object errs.
            """
            super(DataPlotterManager, self).loadProducts(index)
            self.std_rbnspectra        = PlotableSCI_RBNSPECTRA(self.science_frames[index])
            self.std_rbnspectra_extr   = PlotableExtractedScience(self.science_frames[index], fib_id=self.fib_id)
            self.esorex_log_file_name  = re.sub(".*/giraffe/" ,"../../", os.path.dirname(self.science_frames[index].name))+"/log_dir/esorex.log"
            if index >= 0 and index < len(self.err_frames):
                frame = self.err_frames[index]
                self.err = pipeline_product.PipelineProduct(frame)

        # This function creates all the subplots. It is responsible for the
        # plotting layouts.
        # There can different layouts, depending on the availability of data
        # Note that subplot(I,J,K) means the Kth plot in a IxJ grid
        # Note also that the last one is actually a box with text, no graphs.
        def addSubplots(self, figure):
            self.list_subplot_image     = []
            if self.science_found == True and self.err_found == True:
                gs = matplotlib.gridspec.GridSpec(100,1)
                self.subplot_std_rbnspectra       = figure.add_subplot(gs[0:58,0])
                self.subplot_std_rbnspectra_extr  = figure.add_subplot(gs[70:92,0])
                self.subplot_txtinfo       = figure.add_subplot(gs[95:100,0])
            else : 
                self.subtext_nodata      = figure.add_subplot(1,1,1)
            self.setupFigure(figure)

        # This is the function that makes the plots.
        # Add new plots or delete them using the given scheme.
        # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
        # It is mandatory to add a tooltip variable to each subplot.
        # One might be tempted to merge addSubplots() and plotProductsGraphics().
        # There is a reason not to do it: addSubplots() is called only once at
        # startup, while plotProductsGraphics() is called always there is a
        # resize.
        def plotProductsGraphics(self):

            if self.science_found == True and self.err_found == True:
                
                self.plot_std_rbnspectra()
                self.plot_std_rbnspectra_extr()
                
                #Additional text info
                self.showTextInfo()

            else :
                #Data not found info
                self.showNoData()

        def plot_std_rbnspectra(self) :
            title_std_rbnspectra   = 'Standard: Rebinned spectra [STD_RBNSPECTRA]\nCosmic Ray cleaning: %s' %(self.std_rbnspectra.rbnspectra.readKeyword('CMETHOD'))
            tooltip_std_rbnspectra = """Standard: rebinned spectra.\nMiddle-button click on a fibre to display it in the window below."""
            self.std_rbnspectra.plot(self.subplot_std_rbnspectra, title_std_rbnspectra,
                                 tooltip_std_rbnspectra)

        def plot_std_rbnspectra_extr(self) :
            title_std_rbnspectra_extr   = 'Rebinned spectra of OBJECT, S/N = 0.'
            tooltip_std_rbnspectra_extr = """Standard: rebinned spectra."""
            self.std_rbnspectra_extr.plot(self.subplot_std_rbnspectra_extr, title_std_rbnspectra_extr,
                                 tooltip_std_rbnspectra_extr)

        def showTextInfo(self) :
          self.subplot_txtinfo.cla()
          self.subplot_txtinfo.set_axis_off()
          try:
            mode = self.std_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS SLITS ID')
          except:
            mode = 'unknown'
          try:
            filter_name = self.std_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS FILT NAME')
          except:
            filter_name = 'unknown'
          try:
            grat_wlen = self.std_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS GRAT WLEN')
          except:
            grat_wlen = 'unknown'
          try:
            raw1_name = self.std_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO PRO REC1 RAW1 NAME')
          except:
            raw1_name = 'unknown'
          self.subplot_txtinfo.text(0.1, 0., 'DPid RAW1: %s\nDPR: %s :: %s :: %s\nMode/Setting/WLen: %s/%s/%s'
                                   %(raw1_name,
                                     self.DPR_CATG,self.DPR_TYPE,self.DPR_TECH,
                                     mode,filter_name,str(grat_wlen)
                                    ),
                                   ha='left', va='top', weight='bold')
          try:
            slit = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS SLIT NAME')
            self.subplot_txtinfo.text(0.5, 0., 'Slit name (LSS): '+slit, 
                                   ha='left', va='center', weight='bold')
          except:
            pass
          try:
            checksum = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MOS CHECKSUM')
            self.subplot_txtinfo.text(0.5, 0., 'Slits checksum (MOS): '+str(checksum), 
                                   ha='left', va='center', weight='bold')
          except:
            pass
          try:
            mask_id = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MASK ID')
            self.subplot_txtinfo.text(0.5, 0., 'Mask id (MXU): '+mask_id, 
                                   ha='left', va='center', weight='bold')
          except:
            pass
          try:
            target_name = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO OBS TARG NAME')
            self.subplot_txtinfo.text(0.1, 0.6, 'Target name: '+target_name, 
                                   ha='left', va='center', weight='bold')
          except:
            pass

        def showNoData(self) :
          #Data not found info
          self.subtext_nodata.set_axis_off()
          self.text_nodata = 'Standard data not found in the products (PRO.CATG=STD_RBNSPECTRA)'
          self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                                   ha='left', va='center', alpha=1.0)
          self.subtext_nodata.tooltip='Standard data not found in the products'

        def plotWidgets(self) :
          widgets = list()
          if self.std_rbnspectra is not None :
            # Clickable subplot
            self.clickablesmapped = InteractiveClickableSubplot(
              self.subplot_std_rbnspectra, self.setExtractedObject)
            widgets.append(self.clickablesmapped)
          return widgets

        def setExtractedObject(self, point) :
            obj = self.std_rbnspectra.getObjectInPosition(point.xdata+0.5)
            if obj != -1 :
                self.std_rbnspectra_extr.selectObject(obj)
                self.fib_id=obj
            self.plot_std_rbnspectra_extr()

    
        # This function specifies which are the parameters that should be presented
        # in the window to be edited.
        # Note that the parameter has to be also in the in_sop port (otherwise it
        # won't appear in the window)
        # The descriptions are used to show a tooltip. They should match one to one
        # with the parameter list
        # Note also that parameters have to be prefixed by the 'recipe name:'
        def setInteractiveParameters(self):
            paramList = list()
            paramList.append(reflex.RecipeParameter('gistandard','bsremove-method',group='Bkg Sub',description='Backgruond subtraction method'))
            paramList.append(reflex.RecipeParameter('gistandard','bsremove-yorder',group='Bkg Sub',description='Background subtraction yorder of 2-D polynomial fit'))
            paramList.append(reflex.RecipeParameter('gistandard','extr-method',group='Extr',description="Extraction method: 'SUM', 'HORNE' or 'OPTIMAL'. <SUM | OPTIMAL | HORNE> [SUM]"))
            paramList.append(reflex.RecipeParameter('gistandard','flat-apply',group='Flat',description="Apply or not the flat field correction. [TRUE]"))
            return paramList

        def setWindowHelp(self):
            help_text = """
This is an interactive window which help to assess the quality of the execution of a recipe.
"""
            return help_text

        def setWindowTitle(self):
            title = 'Pipeline Interactive Window'
            return title

except ImportError:
    DataPlotterManager = None


if __name__ == '__main__':
    runInteractiveApp(DataPlotterManager)



# import the needed modules
try:
  import reflex
  import matplotlib.gridspec
  import reflex_plot_widgets
  import os
  import_sucess = True

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def __init__(self):
        #self.xrc_file=os.path.join(os.path.dirname(__file__),'giraf_reflex_interactive_gui.xrc')
        self.xrc_file=None
    
    def readFitsData(self, fitsFiles):
      import os.path, glob
      for F in glob.glob('../../gimasterflat_1/latest/*.fits') :
          HDUlist=pyfits.open(F)
          fitsFiles.append(reflex.FitsFile(os.path.abspath(F), HDUlist[0].header['HIERARCH ESO PRO CATG'], None, None))
          HDUlist.close()

      #Initialise the objects to read/display
      self.masterflat     = None
      self.ff_loccentroid = None
      self.ff_psfcentroid = None
      self.ff_raw = None
      self.objid = 1

      #Read all the products
      frames = dict()
      for frame in fitsFiles :
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      # re-define plotting functions to enable callbacks
      self._add_subplots = self._add_subplots
      self._plot = self._data_plot

      #for inst_mode in ('MEDUSA', 'IFU', 'ARGUS') :
      self.radio_button_opts = {}
      if 'MASTER_FIBER_FLAT' in frames:
          self.masterflat   = PlotableFlat(frames["MASTER_FIBER_FLAT"])
          self.radio_button_opts = {'Master FLAT':0}
          if 'FF_LOCCENTROID' in frames :
              self.ff_loccentroid = frames['FF_LOCCENTROID']
              self.masterflatWithLOC   = PlotableFlatWithLOCcentroid(frames["MASTER_FIBER_FLAT"],self.ff_loccentroid)
              self.radio_button_opts = {'Localization':0}
              if 'FF_PSFCENTROID' in frames :
                  self.ff_psfcentroid = frames['FF_PSFCENTROID']
                  self.masterflatWithPSF   = PlotableFlatWithPSFcentroid(frames["MASTER_FIBER_FLAT"],self.ff_psfcentroid)
                  self.radio_button_opts = {'Localization':0,'PSF':1}
          if 'FIBER_FLAT' in frames:
              self.ff_raw = frames['FIBER_FLAT']
              HDUlist=pyfits.open(self.ff_raw.name)
              self.DPR_CATG = HDUlist[0].header['HIERARCH ESO DPR CATG']
              self.DPR_TYPE = HDUlist[0].header['HIERARCH ESO DPR TYPE']
              self.DPR_TECH = HDUlist[0].header['HIERARCH ESO DPR TECH']
              HDUlist.close()

          # Set the initial radio button selections (value 0)
          self.radio_button_label = [key for key, value in self.radio_button_opts.iteritems() if value == 0][0]
      else:
          # Set the plotting functions to NODATA ones
          self._add_subplots = self._add_nodata_subplots
          self._plot = self._nodata_plot


    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
        self._add_subplots(figure)

    def _add_subplots(self, figure):
      if self.masterflat is not None :
        if self.ff_loccentroid is not None :
            gs = matplotlib.gridspec.GridSpec(100,100)
            self.axradiobutton         = figure.add_subplot(gs[0:5,0:20])
            self.subplot_masterflat    = figure.add_subplot(gs[7:45,0:100])
            self.subplot_psfcentroid   = figure.add_subplot(gs[54:92,0:100])
            self.subplot_txtinfo       = figure.add_subplot(gs[95:100,0:100])
#            self.subplot_txtinfo       = figure.widgets.Widget(gs[95:100,0:100])
        else :
            gs = matplotlib.gridspec.GridSpec(100,100)
            self.subplot_masterflat    = figure.add_subplot(gs[0:72,0:100])
            self.subplot_psfcentroid   = figure.add_subplot(gs[80:92,0:100])
            self.subplot_txtinfo       = figure.add_subplot(gs[95:100,0:100])

    def _add_nodata_subplots(self, figure):
        self.subtext_nodata      = figure.add_subplot(1,1,1)


    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
        self._plot()

    def _data_plot(self):
      if self.masterflat is not None :

        #MasterFlat + ff_loccentroid (if existing...)
        if self.ff_loccentroid is not None :
            self.plotMasterFlatWithLOCcentroid()
            #MasterFlat + ff_psfcentroid (if existing...)
            if self.ff_psfcentroid is not None :
                if (self.radio_button_opts[self.radio_button_label] == 0) :
                    self.plotMasterFlatWithLOCcentroid()
                elif (self.radio_button_opts[self.radio_button_label] == 1) :
                    self.plotMasterFlatWithPSFcentroid()
            else:
                self.showNoPSF()
        else:
            self.plotMasterFlat()
            self.showNoLOC()

        #Additional text info
        self.showTextInfo()

    def _nodata_plot(self):
        #else :
        #Data not found info
        self.showNoData()
  
    def plotMasterFlat(self) :
        title_masterflat   = 'Master Flat'
        tooltip_masterflat = """Master Flat.
The master flat generated by the gimasterflat recipe, apparently processing failed, probably the recipe was
unable to find the epected number of fibres."""
        self.masterflat.plot(self.subplot_masterflat, title_masterflat,
                             tooltip_masterflat)

    def plotMasterFlatWithLOCcentroid(self) :
        title_masterflat   = 'Master Flat with LOCCENTROID'
        tooltip_masterflat = """Master Flat.
The centroids of the fibre location overlaid on the master flat."""
        self.masterflatWithLOC.plot(self.subplot_masterflat, title_masterflat,
                             tooltip_masterflat)
        title_masterflat   = 'Master Flat cross cut with LOCCENTROID'
        tooltip_masterflat = """Master Flat.
The centroids of the fibre location overlaid on a cross section of the master flat."""
        self.masterflatWithLOC.plotXsection(self.subplot_psfcentroid, title_masterflat,
                             tooltip_masterflat)

    def plotMasterFlatWithPSFcentroid(self) :
        title_masterflat   = 'Master Flat with PSFCENTROID'
        tooltip_masterflat = """Master Flat.
The centroids of the PSF fitting overlaid on the master flat."""
        self.masterflatWithPSF.plot(self.subplot_masterflat, title_masterflat,
                             tooltip_masterflat)
        title_masterflat   = 'Master Flat cross cut with PSFCENTROID'
        tooltip_masterflat = """Master Flat.
The centroids of the PSF fitting overlaid on a cross section of the master flat."""
        self.masterflatWithPSF.plotXsection(self.subplot_psfcentroid, title_masterflat,
                             tooltip_masterflat)

    def showNoLOC(self) :
      #Data not found info
      self.subplot_psfcentroid.set_axis_off()
      self.text_nodata = 'Recipe FAILED, could not find product FF_LOCCENTROID.'
      self.subplot_psfcentroid.text(0.5, 0.5, self.text_nodata, color='red', fontsize=18,
                               ha='center', va='center', alpha=1.0)
      self.subplot_psfcentroid.tooltip="""The gimasterflat recipe has apparently failed to even localaize the fibres.
Probably it failed to locate the expected number of fibres, try adjusting fiber-splist on the 'Fibre selection' tab, and/or the
parameters on the 'Location' tab."""

    def showNoPSF(self) :
      #Data not found info
      self.subplot_psfcentroid.set_axis_off()
      self.text_nodata = 'Recipe FAILED, found FF_LOCCENTROID product,\nbut could not find product FF_PSFCENTROID.'
      self.subplot_psfcentroid.text(0.5, 0.5, self.text_nodata, color='red', fontsize=18,
                               ha='center', va='center', alpha=1.0)
      self.subplot_psfcentroid.tooltip="""The gimasterflat recipe has apparently failed to buid the PSF models of fibres.
Possibly the localization found the right number of fibres, but failed to trace them correctly. Try adjusting the
parameters on the 'Location' and 'PSF' tab."""


    def showTextInfo(self) :
      self.subplot_txtinfo.set_axis_off()
      try:
        mode = self.masterflat.flat.readKeyword('HIERARCH ESO INS SLITS ID')
      except:
        mode = 'unknown'
      try:
        filter_name = self.masterflat.flat.readKeyword('HIERARCH ESO INS FILT NAME')
      except:
        filter_name = 'unknown'
      try:
        grat_wlen = self.masterflat.flat.readKeyword('HIERARCH ESO INS GRAT WLEN')
      except:
        grat_wlen = 'unknown'
      try:
        raw1_name = self.masterflat.flat.readKeyword('HIERARCH ESO PRO REC1 RAW1 NAME')
      except:
        raw1_name = 'unknown'

      self.subplot_txtinfo.text(0.1, 0., 'DPid RAW1: %s\nDPR: %s :: %s :: %s\nMode/Setting/WLen: %s/%s/%s'
                               %(raw1_name,
                                 self.DPR_CATG,self.DPR_TYPE,self.DPR_TECH,
                                 mode,filter_name,str(grat_wlen)
                                ),
                               ha='left', va='top', weight='bold')

      try:
        slit = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS SLIT NAME')
        self.subplot_txtinfo.text(0.5, 0., 'Slit name (LSS): '+slit, 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        checksum = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MOS CHECKSUM')
        self.subplot_txtinfo.text(0.5, 0., 'Slits checksum (MOS): '+str(checksum), 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        mask_id = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MASK ID')
        self.subplot_txtinfo.text(0.5, 0., 'Mask id (MXU): '+mask_id, 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        target_name = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO OBS TARG NAME')
        self.subplot_txtinfo.text(0.1, 0.6, 'Target name: '+target_name, 
                               ha='left', va='center', weight='bold')
      except:
        pass

    def showNoData(self) :
      #Data not found info
      self.subtext_nodata.set_axis_off()
      self.text_nodata = 'Master Flat data not found in the products (PRO.CATG=)'
      self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                               ha='left', va='center', alpha=1.0)
      self.subtext_nodata.tooltip='Master Flat data not found in the products'

    def plotWidgets(self) :
      widgets = list()
      if False and self.masterflat is not None :
        # Clickable subplot
        self.clickablesmapped = InteractiveClickableSubplot(
          self.subplot_masterflat, self.setExtractedObject)
        widgets.append(self.clickablesmapped)
        
      if self.masterflat is not None :
          if self.ff_loccentroid is not None :
              # Radio button
              self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.axradiobutton, 
                                                                     self.setRadioCallback, 
                                                                     [key for key,value in sorted(self.radio_button_opts.items(),key= lambda k: k[1])],
                                                                     self.radio_button_opts.get(self.radio_button_label),
                                                                     title='Select item to display')
              widgets.append(self.radiobutton)

      return widgets

    def setRadioCallback(self, label) :

        # Only do something if user changes the button
        if (label != self.radio_button_label):
            self.radio_button_label = label
            self._plot()

    def setExtractedObject(self, point) :
      obj = self.masterflat.getObjectInPosition(point.ydata)
      if obj != -1 :
        self.sci_extracted.selectObject(obj)
      self.plotSciExtracted()
        
                
    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter('gimasterflat','fiber-splist',group='Geom',description='Index list of spectra to use for localization (e.g. 2,10,30-40,55)'))
      paramList.append(reflex.RecipeParameter('gimasterflat','bsremove-method',group='Bkg Sub',description='Backgruond subtraction method'))
      paramList.append(reflex.RecipeParameter('gimasterflat','bsremove-yorder',group='Bkg Sub',description='Background subtraction yorder of 2-D polynomial fit'))
      paramList.append(reflex.RecipeParameter('gimasterflat','sloc-start',group='LOC',description='Bin along x-axis. [-1]'))
      paramList.append(reflex.RecipeParameter('gimasterflat','sloc-binsize',group='LOC',description='Initial localization detection xbin size. [-1]'))
      paramList.append(reflex.RecipeParameter('gimasterflat','sloc-retries',group='LOC',description='Initial localization detection xbin retries. [10]'))
      paramList.append(reflex.RecipeParameter('gimasterflat','sloc-noise',group='LOC',description='Threshold multiplier. [7.0]'))
      paramList.append(reflex.RecipeParameter('gimasterflat','psf-binsize',group='PSF',description='Size of bin along dispersion axis. [64]'))

      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user will interact with the GIRAF flat-field reduction"""
      return help_text

    def setWindowTitle(self):
      title = 'GIRAF Interactive masterflat reduction'
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import GIRAF reflex modules
  from giraf_plot_common import *

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()



try:
  import numpy
  from pipeline_display import *
  from pipeline_product import *
  from reflex_plot_widgets import *
  from numpy.polynomial import Polynomial
  numpy.seterr(invalid='ignore')
  import giraf_der_snr
except ImportError:
  donothing=1


## Potentially interesting code here...
## http://python4esac.github.io/plotting/specnorm.html

## These are the GIRAFFE routines, hacked from the FORS ones,
## see below for the remaining FORS ones as examples...
class PlotableFlat(object) :
  def __init__(self, fits_flat):
    self.flat      = PipelineProduct(fits_flat)
    self.flatdisp  = ImageDisplay()
    self.loadFromFits()

  def loadFromFits(self) :
    #Reading the flat image
    self.flat.readImage()

  def plot(self, subplot, title, tooltip):
    self.flatdisp.setLabels('X [pix]', 'Y [pix]')
    self.flatdisp.display(subplot, title, tooltip, self.flat.image)

################################################################################

class PlotableFlatWithLOCcentroid(object) :
  def __init__(self, fits_flat, fits_ff_loccentroid):
    self.Xsectplot = -1
    self.flat      = PipelineProduct(fits_flat)
    self.ff_loccentroid = None
    if fits_ff_loccentroid is not None:
      self.ff_loccentroid = PipelineProduct(fits_ff_loccentroid)
    self.flatdisp  = ImageDisplay()
    self.Xsectplot = SpectrumDisplay()
    self.loadFromFits()
    
    self.NAXIS1=self.flat.all_hdu[0].header['NAXIS1']
    self.NAXIS2=self.flat.all_hdu[0].header['NAXIS2']
    # Params are available via the interactive_app.getCurrentParameterValue()
    # but I can't figure out how to access them from here... so I need to
    # make my own param dictionary...
    i=0
    pFound=True
    self.params = {}
    self.params[self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d NAME' %(i+1)]] = self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d VALUE' %(i+1)]
    while pFound :
        try :
            self.params[self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d NAME' %(i+1)]] = self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d VALUE' %(i+1)]
            i+=1
        except :
            pFound=False
    self.sloc_start=int(self.params['sloc-start'])
    self.setXsectRow(self.sloc_start)

  def loadFromFits(self) :
    #Reading the flat image
    self.flat.readImage()
    if self.ff_loccentroid is not None:
      self.ff_loccentroid.readImage()
      self.fibres = self.ff_loccentroid.image.shape[1]

      #Creating the points to plot based on the polynomail traces
      self.ypos_fibres = range(self.ff_loccentroid.image.shape[0])
      self.xpos_fibres = []
      for fibre in numpy.arange(self.fibres) :
        self.xpos_fibres.append(self.ff_loccentroid.all_hdu[0].data[:,fibre]+1.)


  def plot(self, subplot, title, tooltip):
    subplot.cla()
    self.flatdisp.setLabels('X [pix]', 'Y [pix]')
    self.flatdisp.display(subplot, title, tooltip, self.flat.image)
    lss=self.sloc_start
    if lss == -1 :
        lss=int(self.NAXIS2/2)
    if self.ff_loccentroid is not None:
      subplot.autoscale(enable=False)
      ## EVEN number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 0)] :
        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
                     linestyle='solid',color='red')
      ## ODD number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 1)] :
        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
                     linestyle='solid',color='yellow')
      ## Label the subslits...
      for SS in numpy.unique(self.ff_loccentroid.all_hdu[1].data['SSN']):
          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
              xycoords='data', horizontalalignment='center', verticalalignment='top')
      ## Label the fibres...
      j=0
      for i in numpy.unique(self.ff_loccentroid.all_hdu[1].data['FPS']):
        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,lss],lss),
              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
        j+=1
    ## A horizontal line at the sloc-start value...
    subplot.plot([0,self.NAXIS1], [lss,lss],
                 linestyle='solid', color='black', linewidth=2)

  def setXsectRow(self, row):
      self.XsectRow=row
      if row == -1 :
          self.XsectRow=int(self.NAXIS2/2)

  def plotXsection(self, subplot, title, tooltip):
    subplot.cla()
    self.Xsectplot.setLabels('X [pix]', 'Y [ADU]')
    self.Xsectplot.display(subplot, title, tooltip, numpy.arange(self.NAXIS1)+1, self.flat.image[self.XsectRow,:])
    SSylab=subplot.get_ylim()[1]*0.99
    lss=self.sloc_start
    if lss == -1 :
        lss=int(self.NAXIS2/2)
    if self.ff_loccentroid is not None:
      subplot.autoscale(enable=False)
      ## EVEN number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 0)] :
        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
                     linestyle='solid',color='red')
      ## ODD number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN']%2 == 1)] :
        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
                     linestyle='solid',color='yellow')
      ## Label the subslits...
      for SS in numpy.unique(self.ff_loccentroid.all_hdu[1].data['SSN']):
          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_loccentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
              xycoords='data', horizontalalignment='center', verticalalignment='top')
      ## Label the fibres...
      j=0
      for i in numpy.unique(self.ff_loccentroid.all_hdu[1].data['FPS']):
        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,self.XsectRow],0),
              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
        j+=1

################################################################################

class PlotableFlatWithPSFcentroid(object) :
  def __init__(self, fits_flat, fits_ff_psfcentroid):
    self.flat      = PipelineProduct(fits_flat)
    self.ff_psfcentroid = None
    if fits_ff_psfcentroid is not None:
      self.ff_psfcentroid = PipelineProduct(fits_ff_psfcentroid)
    self.flatdisp  = ImageDisplay()
    self.Xsectplot = SpectrumDisplay()
    self.loadFromFits()

    self.NAXIS1=self.flat.all_hdu[0].header['NAXIS1']
    self.NAXIS2=self.flat.all_hdu[0].header['NAXIS2']
    # Params are available via the interactive_app.getCurrentParameterValue()
    # but I can't figure out how to access them from here... so I need to
    # make my own param dictionary...
    i=0
    pFound=True
    self.params = {}
    self.params[self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d NAME' %(i+1)]] = self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d VALUE' %(i+1)]
    while pFound :
        try :
            self.params[self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d NAME' %(i+1)]] = self.flat.all_hdu[0].header['HIERARCH ESO PRO REC1 PARAM%d VALUE' %(i+1)]
            i+=1
        except :
            pFound=False
    self.sloc_start=int(self.params['sloc-start'])
    self.setXsectRow(self.sloc_start)

  def loadFromFits(self) :
    #Reading the flat image
    self.flat.readImage()
    if self.ff_psfcentroid is not None:
      self.ff_psfcentroid.readImage()
      self.fibres = self.ff_psfcentroid.image.shape[1]

      #Creating the points to plot based on the polynomail traces
      self.ypos_fibres = range(self.ff_psfcentroid.image.shape[0])
      self.xpos_fibres = []
      for fibre in range(self.fibres) :
        self.xpos_fibres.append(self.ff_psfcentroid.all_hdu[0].data[:,fibre]+1.)


  def plot(self, subplot, title, tooltip):
    subplot.cla()
    self.flatdisp.setLabels('X [pix]', 'Y [pix]')
    self.flatdisp.display(subplot, title, tooltip, self.flat.image)
    lss=self.sloc_start
    if lss == -1 :
        lss=int(self.NAXIS2/2)
    if self.ff_psfcentroid is not None:
      subplot.autoscale(enable=False)
      ## EVEN number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 0)] :
        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
                     linestyle='solid',color='blue')
      ## ODD number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 1)] :
        subplot.plot(self.xpos_fibres[fibre], self.ypos_fibres,
                     linestyle='solid',color='green')
      ## Label the subslits...
      for SS in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['SSN']):
          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,lss]),self.NAXIS2),
              xycoords='data', horizontalalignment='center', verticalalignment='top')
      ## Label the fibres...
      j=0
      for i in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['FPS']):
        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,lss],lss),
              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
        j+=1
    ## A horizontal line at the sloc-start value...
    subplot.plot([0,self.NAXIS1], [lss,lss],
                 linestyle='solid', color='black', linewidth=2)

  def setXsectRow(self, row):
      self.XsectRow=row
      if row == -1 :
          self.XsectRow=int(self.NAXIS2/2)

  def plotXsection(self, subplot, title, tooltip):
    subplot.cla()
    self.Xsectplot.setLabels('X [pix]', 'Y [ADU]')
    self.Xsectplot.display(subplot, title, tooltip, numpy.arange(self.NAXIS1)+1, self.flat.image[self.XsectRow,:])
    SSylab=subplot.get_ylim()[1]*0.99
    lss=self.sloc_start
    if lss == -1 :
        lss=int(self.NAXIS2/2)
    if self.ff_psfcentroid is not None:
      subplot.autoscale(enable=False)
      ## EVEN number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 0)] :
        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
                     linestyle='solid',color='blue')
      ## ODD number subslits...
      for fibre in numpy.arange(self.fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN']%2 == 1)] :
        subplot.axvline(self.xpos_fibres[fibre][self.XsectRow],
                     linestyle='solid',color='green')
      ## Label the subslits...
      for SS in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['SSN']):
          subplot.annotate(str(SS), xy=(numpy.mean(numpy.array(self.xpos_fibres)[numpy.where(self.ff_psfcentroid.all_hdu[1].data['SSN'] == SS)][:,self.XsectRow]),SSylab),
              xycoords='data', horizontalalignment='center', verticalalignment='top')
      ## Label the fibres...
      j=0
      for i in numpy.unique(self.ff_psfcentroid.all_hdu[1].data['FPS']):
        subplot.annotate(str(i), xy=(numpy.array(self.xpos_fibres)[j,self.XsectRow],0),
              xycoords='data', horizontalalignment='center', verticalalignment='bottom', rotation=90.)
        j+=1


################################################################################

class PlotableARC_RBNSPECTRA(object) :
  def __init__(self, fits_arc_rbnspectra):
    self.rbnspectra      = PipelineProduct(fits_arc_rbnspectra)
    self.rbnspectradisp  = ImageDisplay()
    self.loadFromFits()

  def loadFromFits(self) :
    #Reading the flat image
    self.rbnspectra.readImage()

  def plot(self, subplot, title, tooltip):
    self.rbnspectradisp.setLabels('Fibre', 'Y [pix]')
    self.rbnspectradisp.display(subplot, title, tooltip, self.rbnspectra.image)



################################################################################

class PlotableSCI_RBNSPECTRA(object) :
  def __init__(self, fits_sci_rbnspectra):
    self.rbnspectra      = PipelineProduct(fits_sci_rbnspectra)
    self.rbnspectradisp  = ImageDisplay()
    self.loadFromFits()

  def loadFromFits(self) :
    #Reading the flat image
    self.rbnspectra.readImage()

  def plot(self, subplot, title, tooltip):
    self.rbnspectradisp.setLabels('Fibre', 'Y [pix]')
    self.rbnspectradisp.display(subplot, title, tooltip, self.rbnspectra.image)

  def getObjectInPosition(self, xpos) :
    return int(xpos)

################################################################################

class PlotableExtractedScience :
    def __init__(self, fits_extractedscience, fib_id=-1):
        self.fib_id = fib_id
        self.extractedscience  = PipelineProduct(fits_extractedscience)
        self.spectrumdisplay   = SpectrumDisplay()
        self.loadFromFits()

    def loadFromFits(self) :
        #Reading the flat image
        self.extractedscience.readImage()
        self.nobj   = self.extractedscience.image.shape[1]
        self.crpix  = self.extractedscience.readKeyword('CRPIX2', 0)
        self.crval  = self.extractedscience.readKeyword('CRVAL2', 0)
        self.cdelt  = self.extractedscience.readKeyword('CDELT2', 0)
        self.bunit  = self.extractedscience.readKeyword('BUNIT', 0)
        self.nwave  = self.extractedscience.image.shape[0]
        self.isARG  = self.extractedscience.readKeyword('HIERARCH ESO INS MODE', 0) == 'ARG'
        self.wave   = numpy.arange(1, self.nwave+1, 1)
        self.wave   = (self.wave - self.crpix) * self.cdelt + self.crval
        self.selectHighestSNR()
        self.setFluxSelected()

    def selectBrightest(self):
        if self.nobj == 1:
            self.fib_id = 1
        median = 0
        for obj in range(self.nobj) :
            new_median = numpy.median(self.extractedscience.image[:,obj])
            if new_median > median :
                median = new_median
                self.fib_id = obj + 1

    def selectHighestSNR(self):
        self.SNRarr=numpy.zeros(self.nobj)
        if self.nobj == 1:
            self.fib_id = 1
        for obj in range(self.nobj) :
            self.SNRarr[obj] = giraf_der_snr.DER_SNR(self.extractedscience.image[:,obj])
        if(self.fib_id == -1) : # Select Highest SNR
            self.fib_id = numpy.argsort(self.SNRarr)[-1]+1

    def setFluxSelected(self) :
        self.flux = self.extractedscience.image[:,self.fib_id-1]
        self.OBJECT = self.extractedscience.all_hdu[1].data['OBJECT'][self.fib_id-1]
        self.FPS = self.extractedscience.all_hdu[1].data['FPS'][self.fib_id-1]
        self.SSN = self.extractedscience.all_hdu[1].data['SSN'][self.fib_id-1]
        if self.isARG and not self.extractedscience.all_hdu[1].data['TYPE'][self.fib_id-1] == 'S' :
            self.OBJECT = self.extractedscience.all_hdu[0].header['HIERARCH ESO OBS TARG NAME']
        self.OBJECT_SNR = self.SNRarr[self.fib_id-1]

    def selectObject(self, fib_id):
        self.fib_id = fib_id
        self.setFluxSelected()

    def plot(self, subplot, title, tooltip):
        subplot.cla()
        wlUnit=""
        try :
            wlUnit=" ["+self.extractedscience.all_hdu[0].header['CUNIT2']+"]"
        except :
            pass
        ltitle='Rebinned spectrum of %s, S/N = %5.2f [FPS/SSN=%d/%d, col=%d]' %(self.OBJECT, self.OBJECT_SNR, self.FPS, self.SSN, self.fib_id)
        self.spectrumdisplay.setLabels('Wavelength'+wlUnit, 'Total Flux ['+self.bunit+']')
        self.spectrumdisplay.display(subplot, ltitle, tooltip, self.wave, self.flux,
                                     autolimits = True)




import os
import re
# import the needed modules
try:
  import reflex
  import matplotlib.gridspec
  import reflex_plot_widgets
  import_sucess = True

#NOTE for developers: 
# -If you want to modify the current script to cope
#  with different parameters, this is the function to modify:
#  setInteractiveParameters()
# -If you want to modify the current script to read different data from
#  the input FITS, this is the function to modify:
#  readFitsData()                  (from class DataPlotterManager) 
# -If you want to modify the current script to modify the plots (using the same
#  data),  this is the function to modify:
#  plotProductsGraphics()          (from class DataPlotterManager)
# -If you want to modify the text that appears in the "Help" button,
#  this is the function to modify:
#  setWindowHelp()
# -If you want to modify the title of the window, modify this function:
#  setWindowTitle()


  #This class deals with the specific details of data reading and final plotting.
  class DataPlotterManager:
    # This function will read all the columns, images and whatever is needed
    # from the products. The variables , self.plot_x, self.plot_y, etc...
    # are used later in function plotProductsGraphics().
    # Add/delete these variables as you need (only that plotProductsGraphics()
    # has to use the same names).
    # You can also create some additional variables (like statistics) after
    # reading the files.
    # If you use a control variable (self.xxx_found), you can modify 
    # later on the layout of the plotting window based on the presence of 
    # given input files. 
    # sof contains all the set of frames
    def readFitsData(self, fitsFiles):
      import glob
      for F in glob.glob('../../giwavecalibration_1/latest/*.fits') :
          HDUlist=pyfits.open(F)
          fitsFiles.append(reflex.FitsFile(os.path.abspath(F), HDUlist[0].header['HIERARCH ESO PRO CATG'], None, None))
          HDUlist.close()

      #Initialise the objects to read/display
      self.arc_rbnspectra     = None
      self.objid = 1

      #Read all the products
      frames = dict()
      for frame in fitsFiles :
        if frame == '' :
          continue
        category = frame.category
        frames[category] = frame

      #for inst_mode in ('MEDUSA', 'IFU', 'ARGUS') :
      if 'ARC_RBNSPECTRA' in frames:
          self.arc_rbnspectra   = PlotableARC_RBNSPECTRA(frames["ARC_RBNSPECTRA"])
          self.esorex_log_file_name  = re.sub(".*/giraffe/" ,"../../", os.path.dirname(frames["ARC_RBNSPECTRA"].name))+"/log_dir/esorex.log"
          if 'ARC_SPECTRUM' in frames:
              self.ff_raw = frames['ARC_SPECTRUM']
              HDUlist=pyfits.open(self.ff_raw.name)
              self.DPR_CATG = HDUlist[0].header['HIERARCH ESO DPR CATG']
              self.DPR_TYPE = HDUlist[0].header['HIERARCH ESO DPR TYPE']
              self.DPR_TECH = HDUlist[0].header['HIERARCH ESO DPR TECH']
              HDUlist.close()

    # This function creates all the subplots. It is responsible for the plotting 
    # layouts. 
    # There can different layouts, depending on the availability of data
    # Note that subplot(I,J,K) means the Kth plot in a IxJ grid 
    # Note also that the last one is actually a box with text, no graphs.
    def addSubplots(self, figure):
      if self.arc_rbnspectra is not None :
          gs = matplotlib.gridspec.GridSpec(100,1)
          self.subplot_arc_rbnspectra  = figure.add_subplot(gs[0:92,0])
          self.subplot_txtinfo       = figure.add_subplot(gs[95:100,0])
      else : 
        self.subtext_nodata      = figure.add_subplot(1,1,1)
          
    # This is the function that makes the plots.
    # Add new plots or delete them using the given scheme.
    # The data has been already stored in self.plot_x, self.plot_xdif, etc ...
    # It is mandatory to add a tooltip variable to each subplot.
    # One might be tempted to merge addSubplots() and plotProductsGraphics().
    # There is a reason not to do it: addSubplots() is called only once at
    # startup, while plotProductsGraphics() is called always there is a resize.
    def plotProductsGraphics(self):
      if self.arc_rbnspectra is not None :

        self.plot_arc_rbnspectra()

        #Additional text info
        self.showTextInfo()

      else :
        #Data not found info
        self.showNoData()
  
    def plot_arc_rbnspectra(self) :
        title_arc_rbnspectra   = 'Wavelength calibration: Rebinned spectra [ARC_RBNSPECTRA] '
        tooltip_arc_rbnspectra = """Wavelength calibration: rebinned spectra.
Should be smooth."""
        self.arc_rbnspectra.plot(self.subplot_arc_rbnspectra, title_arc_rbnspectra,
                             tooltip_arc_rbnspectra)

    def showTextInfo(self) :
      self.subplot_txtinfo.set_axis_off()
      try:
        mode = self.arc_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS SLITS ID')
      except:
        mode = 'unknown'
      try:
        filter_name = self.arc_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS FILT NAME')
      except:
        filter_name = 'unknown'
      try:
        grat_wlen = self.arc_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO INS GRAT WLEN')
      except:
        grat_wlen = 'unknown'
      try:
        raw1_name = self.arc_rbnspectra.rbnspectra.readKeyword('HIERARCH ESO PRO REC1 RAW1 NAME')
      except:
        raw1_name = 'unknown'
      self.subplot_txtinfo.text(0.1, 0., 'DPid RAW1: %s\nDPR: %s :: %s :: %s\nMode/Setting/WLen: %s/%s/%s'
                               %(raw1_name,
                                 self.DPR_CATG,self.DPR_TYPE,self.DPR_TECH,
                                 mode,filter_name,str(grat_wlen)
                                ),
                               ha='left', va='top', weight='bold')
      try:
        slit = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS SLIT NAME')
        self.subplot_txtinfo.text(0.5, 0., 'Slit name (LSS): '+slit, 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        checksum = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MOS CHECKSUM')
        self.subplot_txtinfo.text(0.5, 0., 'Slits checksum (MOS): '+str(checksum), 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        mask_id = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO INS MASK ID')
        self.subplot_txtinfo.text(0.5, 0., 'Mask id (MXU): '+mask_id, 
                               ha='left', va='center', weight='bold')
      except:
        pass
      try:
        target_name = self.sci_extracted.extractedscience.readKeyword('HIERARCH ESO OBS TARG NAME')
        self.subplot_txtinfo.text(0.1, 0.6, 'Target name: '+target_name, 
                               ha='left', va='center', weight='bold')
      except:
        pass

    def showNoData(self) :
      #Data not found info
      self.subtext_nodata.set_axis_off()
      self.text_nodata = 'Wave calibration data not found in the products (PRO.CATG=)'
      self.subtext_nodata.text(0.1, 0.6, self.text_nodata, color='#11557c', fontsize=18,
                               ha='left', va='center', alpha=1.0)
      self.subtext_nodata.tooltip=' data not found in the products'

    def plotWidgets(self) :
      widgets = list()
      if False and self.arc_rbnspectra is not None :
        # Clickable subplot
        self.clickablesmapped = InteractiveClickableSubplot(
          self.subplot_arc_rbnspectra, self.setExtractedObject)
        widgets.append(self.clickablesmapped)
      return widgets

    def setExtractedObject(self, point) :
      obj = self.arc_rbnspectra.getObjectInPosition(point.ydata)
      if obj != -1 :
        self.sci_extracted.selectObject(obj)
      self.plotSciExtracted()
        

    # This function specifies which are the parameters that should be presented
    # in the window to be edited.
    # Note that the parameter has to be also in the in_sop port (otherwise it 
    # won't appear in the window) 
    # The descriptions are used to show a tooltip. They should match one to one
    # with the parameter list 
    # Note also that parameters have to be prefixed by the 'recipe name:'
    def setInteractiveParameters(self):
      paramList = list()
      paramList.append(reflex.RecipeParameter('giwavecalibration','bsremove-method',group='Bkg Sub',description='Backgruond subtraction method'))
      paramList.append(reflex.RecipeParameter('giwavecalibration','bsremove-yorder',group='Bkg Sub',description='Background subtraction yorder of 2-D polynomial fit'))
      paramList.append(reflex.RecipeParameter('giwavecalibration','wcal-lswidth',group='WCAL',description='wcal-lswidth'))


      return paramList

    def setWindowHelp(self):
      help_text = """
In this window, the user will interact with the GIRAF wavelength calibration reduction"""
      return help_text

    def setWindowTitle(self):
      title = 'GIRAF Interactive wavecalibration reduction'
      return title

except ImportError:
  import_sucess = 'false'
  print "Error importing modules pyfits, wx, matplotlib, numpy"

#This is the 'main' function
if __name__ == '__main__':

  # import reflex modules
  import reflex_interactive_app
  import sys

  # import GIRAF reflex modules
  from giraf_plot_common import *

  # Create interactive application
  interactive_app = reflex_interactive_app.PipelineInteractiveApp(enable_init_sop=True)

  #Check if import failed or not
  if import_sucess == 'false' :
    interactive_app.setEnableGUI('false')

  #Open the interactive window if enabled
  if interactive_app.isGUIEnabled() :
    #Get the specific functions for this window
    dataPlotManager = DataPlotterManager()
    interactive_app.setPlotManager(dataPlotManager)
    interactive_app.showGUI()
  else :
    interactive_app.passProductsThrough()

  # print outputs
  interactive_app.print_outputs()

  sys.exit()



# import the needed modules
try:
    import reflex
    import sys
    import os
    try :
        from astropy.io import fits as pyfits
        import_astropy_io=True
    except:
        import pyfits
        import_astropy_io=False
    import logging
    import PyCosmic
    import_sucess = True

    def pycosmic_multi():

        # get Kepler parameters:
        parameters= {}
        for p in sys.argv[1:]:
            parameters[p.split("=")[0]]= p.split("=")[1]

        #Define one output file for every input file:
        outputs.out_sof = inputs.in_sof
        files = outputs.out_sof.files

        masterbiases={}
        badpixmaps={}
        for file in files:
            if file.category == 'MASTER_BIAS' :
                for p in file.purposes :
                    masterbiases[p]=file
            ## For the moment using the Bad Pixel map trying causes PyCosmic to crash, so don't...
            if file.category == 'BAD_PIXEL_MAP' :
                for p in file.purposes :
                    badpixmaps[p]=file

        #Define PyCosmic parameters:
        args=lambda: None
        args.rdnoise="HIERARCH ESO DET OUT1 RON"
        args.siglim=5.0
        args.fwhm=1.7 #2.5 # 2.0
        args.rlim=0.5 # 1.2
        args.iter=5 #5
        #args.iter=1 #5
        args.replacebox=[5,5]
        args.radius=0
        args.gain="HIERARCH ESO DET OUT1 CONAD"
        args.parallel=True
        args.verbose=True

        outlist=list()
        for file in files:
            #select a file category:
            if file.category == 'SCIENCE' or file.category == 'STD' :
                purpose=file.purposes[0]
                if purpose in masterbiases :
                    try:
                        ## For the moment we are just using the first MASTER_BIAS, but not sure that is correct...
                        bHDU=pyfits.open(masterbiases[purpose].name)
                    except:
                        try:
                            logging.error('Could not open MASTER_BIAS %s' %(masterbiases[0].name))
                        except:
                            pass
                        sys.exit(1)
                if purpose in badpixmaps :
                    try:
                        ## For the moment we are just using the first MASTER_BIAS, but not sure that is correct...
                        bpmHDU=pyfits.open(badpixmaps[purpose].name)
                    except:
                        try:
                            logging.error('Could not open BAD_PIXEL_MAP %s' %(badpixmaps[0].name))
                        except:
                            pass
                        sys.exit(1)
                # define file names:
                raw_file=file.name
                bs_file=parameters["--products-dir"] + "/" + \
                       raw_file.replace(".fits", "_bs.fits").split("/")[-1]
                clean_file=parameters["--products-dir"] + "/" + \
                       raw_file.replace(".fits", "_PC_clean.fits").split("/")[-1]
                mask_file=parameters["--products-dir"] + "/" + \
                       raw_file.replace(".fits", "_PC_mask.fits").split("/")[-1]
                file.name=clean_file

                bs=pyfits.open(raw_file)
                if purpose in masterbiases :
                    bs[0].data=bs[0].data-bHDU[0].data
                    #bs[0].header['BZERO']=0.
                    if 'BZERO' in  bs[0].header :
                        del bs[0].header['BZERO']
                if purpose in badpixmaps :
                    bs.append(bpmHDU[0])
                    bs[-1].header['EXTNAME']='BADPIX'

                try:
                    bs.writeto(bs_file, overwrite=True)
                except:
                    bs.writeto(bs_file, clobber=True)                

                # detCos
                #Detects and removes cosmics from astronomical images based on Laplacian edge 
                #detection scheme combined with a PSF convolution approach (Husemann  et al. in prep.).
           
                #IMPORTANT: 
                #The image and the readout noise are assumed to be in units of electrons.
                #The image also needs to be BIAS subtracted!
                #The gain can be entered to convert the image from ADUs to electros, when this is down already set gain=1.0 as the default.
                # clean cosmics:
                PyCosmic.detCos(bs_file, mask_file, clean_file, args.rdnoise, gain=args.gain, sigma_det=args.siglim,  rlim=args.rlim, iter=args.iter, fwhm_gauss=args.fwhm, replace_box=args.replacebox,    increase_radius=args.radius, parallel=args.parallel, verbose=args.verbose)

                #...the output of PyCosmic is multiplied by the gain, divide it out again...
                clean=pyfits.open(clean_file, 'update')

                clean[0].data=clean[0].data/clean[0].header[args.gain]+bHDU[0].data

                # update header
                clean[0].header.append(("CMETHOD", "PYCOSMIC", "method for bad pixel cleaning"))
                clean[0].header.append(("NFILES", len(files), "number of CR cleaned files"))
                clean[0].header.append(("CRSIGLIM", args.siglim, "PyCosmic signifiance threshold"))
                clean[0].header.append(("CRRLIM", args.rlim, "PyCosmic contrast limit"))
                clean[0].header.append(("CRITER", args.iter, "PyCosmic number of iterations"))
                clean[0].header.append(("CRGFWHM", args.fwhm, "PyCosmic FWHM of convolution kernel"))
                clean[0].header.append(("CRRBOX", str(args.replacebox), "PyCosmic box to computer cleaned value"))
                clean[0].header.append(("CRRADIUS", args.radius, "PyCosmic number of neighboring pixels"))
                clean[0].header.append(('CRRON',clean[0].header['HIERARCH ESO DET OUT1 RON'],'Readout noise'))
                clean[0].header.append(('CRGAIN',clean[0].header['HIERARCH ESO DET OUT1 CONAD'],'Gain'))
                if purpose in masterbiases :
                    clean[0].header.append(('CRMBIAS',os.path.basename(masterbiases[purpose].name)[0:80-len('CRMBIAS')-5],'MASTER_BIAS used'))
                if purpose in badpixmaps :
                    clean[0].header.append(('CRBPM',os.path.basename(badpixmaps[purpose].name)[0:80-len('CRBPM')-5],'BAD_PIX_MAP used'))
                clean.flush()
                clean.close()

                # append remaining extensions from raw files unchanged:
                raw=pyfits.open(raw_file)
                for iext in range(1,len(raw)):
                    if import_astropy_io :
                        pyfits.append(clean_file, raw[iext].data, header=raw[iext].header, checksum=True)
                    else :
                        pyfits.append(clean_file, raw[iext].data, header=raw[iext].header)
                raw.close()
                #clean=pyfits.open(clean_file, 'update')
                #print(numpy.min(clean[0].data[:]),numpy.max(clean[0].data[:]))
                #clean.close()

                # update purpose
                outlist.append((reflex.FitsFile(file.name,file.category,None,[unicode("PC_crclean/")+file.purposes[0]])))

        #send output to EsoReflex:
        newsof = reflex.SetOfFiles(inputs.in_sof.datasetName,outlist)
        outputs.out_sof = newsof
        #parser.write_outputs()
        for file in outlist:
            print("output:", file.name, file.category, file.purposes)


except ImportError:
    import_sucess = False
    print "Error importing one or more of the modules astropy.io, pyfits, PyCosmic"

if __name__ == '__main__':
  
    # Hacked from...
    # example script to process input files and produce one output file for
    # each processed input.

    #send output to EsoReflex:
    parser = reflex.ReflexIOParser()
        
    #Define ports:
    parser.add_option("-i", "--in_sof", dest="in_sof")
    parser.add_output("-o", "--out_sof", dest="out_sof")

    inputs  = parser.get_inputs()
    outputs = parser.get_outputs()

    #Check if import failed or not
    if import_sucess :
        pycosmic_multi()
    else :
        outputs.out_sof = reflex.SetOfFiles(inputs.in_sof.datasetName,list())

    #send output to EsoReflex:
    parser.write_outputs()

    sys.exit()



# import the needed modules
try:
    import reflex 
    import sys
    import os
    try :
        from astropy.io import fits as pyfits
    except:
        import pyfits
    import pyfits
    import_sucess = True


except ImportError:
    import_sucess = False
    print "Error importing one or more of the modules pyfits, astroscrappy"

if __name__ == '__main__':
  
    # Hacked from...
    # example script to process input files and produce one output file for
    # each processed input.

    parser = reflex.ReflexIOParser()

    #Define ports:
    parser.add_option("-i", "--in_sof", dest="in_sof")
    parser.add_output("-o", "--out_sof", dest="out_sof")

    inputs  = parser.get_inputs()
    outputs = parser.get_outputs()

    outputs.out_sof = inputs.in_sof

    print("Hello")
    parser.write_outputs()

    sys.exit()
